/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib;

/**
 * History:
 * 2007-07-13 [KS]  Unterstützung für alte Archivzugriffe entfernt, Änderung
 *                  für den ArchiveRequestHelper, Kommantare
 *
 * $Log: LogosClientDavConnection.java,v $
 * Revision 1.2  2007/07/13 13:46:54  Strey
 * Anpassung Archivzugriff mit Suchfunktion nach jüngstem Datensatz nach Datenzeitstempel bzw. Archivzeit.
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.11  2006/11/13 09:24:10  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.10  2006/04/28 05:48:48  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.9  2006/01/23 14:55:04  Trier
 * Methoden zur ArchivAnfrage um SortierMöglk. erweitert
 *
 * Revision 1.8  2006/01/20 09:17:14  Trier
 * statische Methode zum  Abfragen des Archivs logos-Ars/streambasiertes Ars mit Übergabe DAVConnectionInterface
 *
 * Revision 1.7  2006/01/19 06:25:01  Trier
 * Exceptionbehandlung Methode getArchiveDataUniversal geändert und javadoc für diese Methode
 *
 * Revision 1.6  2006/01/18 14:15:19  Trier
 * Methode getArchiveDataUniversal hinzugefügt, die mit dem Logos und streambasierten Ars arbeiten kann.
 *
 * Revision 1.5  2005/11/30 06:58:29  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/09/13 11:42:37  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/01/21 13:26:15  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2004/05/17 04:59:42  Strey
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:18  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */
import java.io.IOException;
import java.util.ArrayList;
import java.util.Set;

import de.bsvrz.dav.daf.main.ApplicationCloseActionHandler;
import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavParameters;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dav.daf.main.MissingParameterException;
import de.bsvrz.dav.daf.main.archive.ArchiveData;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKind;
import de.bsvrz.dav.daf.main.archive.ArchiveDataKindCombination;
import de.bsvrz.dav.daf.main.archive.ArchiveDataStream;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * <p>
 * Klasse, die eine ClientDavConnection global zur Verf&uuml;gung stellt.
 * </p>
 *
 * @author Ralf Trier
 * @version 1.0
 */
public class LogosClientDavConnection extends ClientDavConnection implements
		ApplicationCloseActionHandler {

	private static final ArchiveDataKindCombination ARS_DATA_KIND_COMBINATION = new ArchiveDataKindCombination(
			ArchiveDataKind.ONLINE, ArchiveDataKind.ONLINE_DELAYED,
			ArchiveDataKind.REQUESTED, ArchiveDataKind.REQUESTED_DELAYED);
	private static ArrayList<ConnectionListener> listenerList = new ArrayList<ConnectionListener>();
	private static LogosClientDavConnection logosClientDavConnection;
	private static ClientDavParameters clientDavParameters;

	public LogosClientDavConnection(final ClientDavParameters parameters)
			throws MissingParameterException, Exception {
		super(parameters);
		logosClientDavConnection = this;
		clientDavParameters = parameters;
	}

	/**
	 * gibt die letzten Parameter zur&uuml;ck, die zum Erzeugen einer Connection
	 * mit Hilfe dieser Klasse benutzt wurde. Ist null, wenn die Connection mit
	 * {@link #setConnection(LogosClientDavConnection clientDavConnection )
	 * setConnection} gesetzt wurde.
	 *
	 * @return ClientDavParameters
	 */
	public ClientDavParameters getParameters() {
		return clientDavParameters;
	}

	public LogosClientDavConnection(final ClientDavParameters parameters,
			final DataModel dataModel) throws Exception {
		super(parameters, dataModel);
		logosClientDavConnection = this;
		clientDavParameters = parameters;
	}

	public Set<ArchiveData> getArchivData(final String soPid, final String atgPid,
			final String aspPid, final long von, final long bis) throws IOException {
		return ArchiveRequestHelper.getArchivData(this, soPid, atgPid, aspPid,
				von, bis);
	}

	public ArchiveDataStream getArchiveStream(final String soPid, final String atgPid,
			final String aspPid, final long von, final long bis) throws IOException,
			IllegalStateException, InterruptedException {
		return ArchiveRequestHelper.getStream(this, soPid, atgPid, aspPid, von,
				bis);
	}

	/**
	 * <p>
	 * Bestimmung des letzten archivierten Datensatzes mit Datenanteil für die
	 * angegebene {@link SystemObject} - {@link DataDescription} Kombination.
	 * </p>
	 *
	 * @param so
	 *            {@link SystemObject} das anzufragenden Objekt
	 * @param descr
	 *            {@link DataDescription} Datenbeschreibung der gesuchten Daten
	 * @param maxHistory
	 *            long der maximale Zeitbereich in ms, bis zu dem vom aktuellen
	 *            Zeitpunkt aus nach einem Datensatz gesucht werden soll
	 * @throws IllegalStateException
	 *             Falls die Archivanfrage nicht erfolgreich war und keine
	 *             Ergebnisdatenströme bestimmt werden können.
	 * @return ResultData
	 */
	public Dataset getLastArchiveDataByArchiveTime(final SystemObject so,
			final DataDescription descr, final long maxHistory)
			throws IllegalStateException {
		return ArchiveRequestHelper.getLastArchiveDataByArchiveTime(this, so,
				descr, maxHistory);
	}

	/**
	 * <p>
	 * Bestimmung des letzten archivierten Datensatzes mit Datenanteil für die
	 * angegebene {@link SystemObject} - {@link DataDescription} Kombination.
	 * </p>
	 *
	 * @param so
	 *            {@link SystemObject} das anzufragenden Objekt
	 * @param descr
	 *            {@link DataDescription} Datenbeschreibung der gesuchten Daten
	 * @param fromTime
	 *            long Beginn Suchbereich
	 * @param toTime
	 *            long Ende Suchbereich
	 * @throws IllegalStateException
	 *             Falls die Archivanfrage nicht erfolgreich war und keine
	 *             Ergebnisdatenströme bestimmt werden können.
	 * @return ResultData
	 */
	public Dataset getLastArchiveDataByDataTime(final SystemObject so,
			final DataDescription descr, final long fromTime, final long toTime)
			throws IllegalStateException {
		return ArchiveRequestHelper.getLastArchiveDataByDataTime(this, so,
				descr, fromTime, toTime);
	}

	/**
	 * gibt die letzte ClientDavConnection zurück oder null, wenn nie eine
	 * Connection durch einen Construtor-Aufruf dieser Klasse erzeugt oder mit
	 * {@link #setConnection(LogosClientDavConnection clientDavConnection )
	 * setConnection} gesetzt wurde.
	 *
	 * @return ClientDavConnection
	 */
	public static LogosClientDavConnection getInstance() {
		return logosClientDavConnection;
	}

	/**
	 * setzt die globale Connection. Die ClientDavParameters sind nicht
	 * verf&uuml;gbar.
	 *
	 * @param clientDavConnection
	 *            LogosClientDavConnection
	 */
	public static void setConnection(
			final LogosClientDavConnection clientDavConnection) {
		logosClientDavConnection = clientDavConnection;
		clientDavParameters = null;
	}

	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	// ALLE DEPRECATED FUNKTIONEN ENTFERNT
	// !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	// /**
	// * Zugriff auf historische Datensätze aus dem lokalen Archivsystem. Alle
	// im
	// * spezifizierten Zeitbereich aktuellen Datensätze der angegebenen
	// * Datenbeschreibung werden für alle übergebenen Objekte aus dem lokalen
	// * Archivsystem abgefragt und zeitlich sortiert zurückgegeben. Zu beachten
	// * ist, daß der je Objekt erste zurückgegebene Datensatz einen Zeitstempel
	// * haben kann, der vor dem angefragten Zeitbereich liegt, wenn er am
	// Anfang
	// * des Zeitbereichs noch aktuell war.
	// *
	// * @param objects SystemObject[]
	// * @param dataDescription DataDescription
	// * @param fromTime long
	// * @param toTime long
	// * @return Dataset[]
	// * @throws IllegalStateException Falls die Archivanfrage nicht erfolgreich
	// * war und die Ergebnisdaten nicht bestimmt oder nicht erfolgreich
	// * &uuml;bertragen werden konnten.
	// * @throws InterruptedException Falls der aufrufende Thread unterbrochen
	// * wurde, während auf die entsprechende Antwortnachricht aus dem
	// * Archivsystem gewartet wurde.
	// * @deprecated
	// */
	// @Deprecated
	// public Dataset[] getArchiveDataUniversal(SystemObject[] objects,
	// DataDescription dataDescription, long fromTime, long toTime) throws
	// IllegalStateException, InterruptedException {
	//
	// return getArchiveDataUniversal(this, objects, dataDescription, fromTime,
	// toTime, ArchiveOrder.BY_DATA_TIME);
	// }
	//
	// /**
	// * @deprecated
	// */
	// @Deprecated
	// public Dataset[] getArchiveDataUniversal(SystemObject[] objects,
	// DataDescription dataDescription, long fromTime, long toTime, ArchiveOrder
	// archiveOrder) throws
	// IllegalStateException, InterruptedException {
	// return getArchiveDataUniversal(this, objects, dataDescription, fromTime,
	// toTime, archiveOrder);
	// }
	//
	// /**
	// * @deprecated
	// */
	// @Deprecated
	// static public Dataset[] getArchiveDataUniversal(ClientDavInterface
	// davCon, SystemObject[] objects, DataDescription dataDescription, long
	// fromTime, long toTime) throws
	// IllegalStateException, InterruptedException {
	// return getArchiveDataUniversal(davCon, objects, dataDescription,
	// fromTime, toTime, ArchiveOrder.BY_DATA_TIME);
	// }
	//
	// /**
	// * getArchiveDataUniversal
	// *
	// * @param davCon ClientDavInterface
	// * @param objects SystemObject[]
	// * @param dataDescription DataDescription
	// * @param fromTime long
	// * @param toTime long
	// * @param archiveOrder ArchiveOrder
	// * @return Dataset[]
	// * @throws IllegalStateException
	// * @throws InterruptedException
	// * @deprecated
	// */
	// @Deprecated
	// static public Dataset[] getArchiveDataUniversal(ClientDavInterface
	// davCon, SystemObject[] objects, DataDescription dataDescription, long
	// fromTime, long toTime, ArchiveOrder archiveOrder) throws
	// IllegalStateException, InterruptedException {
	//
	// System.out.println("getArchiveDataUniversal(ClientDavInterface davCon, SystemObject[] objects, DataDescription dataDescription, long fromTime, long toTime, ArchiveOrder archiveOrder)");
	//
	// // if (isStreamBasedArchive()) {
	// ArrayList<Dataset> resultDataList = new ArrayList<Dataset>();
	// ArchiveRequestManager arsReqManager = davCon.getArchive();
	// ArchiveTimeSpecification arsTimeSpec = new
	// ArchiveTimeSpecification(TimingType.DATA_TIME, false, fromTime, toTime);
	// ArrayList<ArchiveDataSpecification> arsDataSpecList = new
	// ArrayList<ArchiveDataSpecification>();
	// for (int i = 0; i < objects.length; i++) {
	// ArchiveDataSpecification arsDataSpec = new ArchiveDataSpecification(
	// arsTimeSpec, __ArsDataKindCombination, archiveOrder,
	// ArchiveRequestOption.NORMAL, dataDescription,
	// objects[i]);
	// arsDataSpecList.add(arsDataSpec);
	// }
	//
	// ArchiveDataQueryResult results =
	// arsReqManager.request(ArchiveQueryPriority.MEDIUM, arsDataSpecList);
	// System.err.println("results=" + results);
	// ArchiveDataStream[] streams = results.getStreams();
	// System.err.println("streams " + streams);
	// try {
	// for (int i = 0; i < streams.length; i++) {
	// ArchiveData arsData = streams[i].take();
	// while (arsData != null) {
	// resultDataList.add(arsData);
	// arsData = streams[i].take();
	// }
	// }
	// }
	// catch (IOException ex) {
	// throw new
	// IllegalStateException("Es sind Probleme bei der Kommunikation mit dem Archivsystem aufgetreten. Es wurden noch nicht alle Datensätze übertragen",
	// ex);
	// }
	// catch (Exception e) {
	// e.printStackTrace();
	// }
	// return resultDataList.toArray(new Dataset[resultDataList.size()]);
	// // }
	// // else {
	// // try {
	// // return davCon.getArchiveData(objects, dataDescription, fromTime,
	// toTime);
	// // }
	// // catch (DataNotSubscribedException ex1) {
	// // throw new
	// IllegalStateException("Bei der Kommunikation mit der Konfiguration ist ein Fehler.",
	// ex1);
	// // }
	// // catch (Exception ex1) {
	// // throw new IllegalStateException("Archivsystem nicht angemeldet.",
	// ex1);
	// // }
	// //
	// // }
	// }

	public void addListener(final ConnectionListener listener) {
		synchronized (listenerList) {
			if (!listenerList.contains(listener)) {
				listenerList.add(listener);
			}
			this.setCloseHandler(this);
		}
	}

	public void close(final String reason) {
		// __ArsType = ArsType_UNASSIGNED;
		synchronized (listenerList) {
			for (int i = 0; i < listenerList.size(); i++) {
				((ConnectionListener) listenerList.get(i)).disconnected(reason);
			}
		}
		final LogosClientDavConnection davConOld = this;
		logosClientDavConnection = null;
		while (!reconnect()) {
			try {
				Thread.currentThread().sleep(1000 * 30);
			} catch (final InterruptedException ex) {
			}
		}
		logosClientDavConnection.setCloseHandler(logosClientDavConnection);
		synchronized (logosClientDavConnection.listenerList) {
			for (int i = 0; i < logosClientDavConnection.listenerList.size(); i++) {
				((ConnectionListener) logosClientDavConnection.listenerList
						.get(i)).reconnected();
			}
		}
	}

	public static synchronized boolean reconnect() {

		System.out.println("trying to reconnect");
		try {
			final LogosClientDavConnection davCon = new LogosClientDavConnection(
					clientDavParameters);
			davCon.connect();
			System.out.println("connected..erfolgreich");
			davCon.login();
			System.out.println("login..erfolgreich");

			return true;
		} catch (final Throwable ex) {
			ex.printStackTrace();
			return false;
		}
	}

	public static interface ConnectionListener {
		public void disconnected(String reason);

		public void reconnected();
	}

}
