/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper;

import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.IntegerValueRange;
import de.bsvrz.dav.daf.main.config.IntegerValueState;

/**
 * History:
 *
 * $Log: AtlVerkehrsStaerkeStunde.java,v $
 * Revision 1.2  2007/08/22 07:48:44  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2007/07/04 11:43:21  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.6  2007/01/16 10:03:21  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2006/11/29 10:48:27  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2006/11/13 09:24:12  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/21 13:26:16  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2004/03/23 13:14:18  Trier
 * Basis für LogosApps
 *
 * Revision 1.1.1.1  2004/03/23 07:59:52  Trier
 * vrz
 *
 */

/**
 * kapselt die AttributListe atl.verkehrsStaerkeStunde oder
 * atg.langZeitDatenFahrzeugAnzahl.
 */
public class AtlVerkehrsStaerkeStunde {

	private final Data instanceData;

	public static final String ATTR_NAME_WERT = "Wert";
	public static final String ATTR_NAME_STATUS = "Status";
	public static final String ATTR_NAME_GUETE = "Güte";

	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR = "nicht ermittelbar";
	public static final String ZUSTAND_WERT_FEHLERHAFT = "fehlerhaft";
	public static final String ZUSTAND_WERT_NICHT_ERMITTELBAR_FEHLERHAFT = "nicht ermittelbar/fehlerhaft";

	protected boolean _hasState = true;

	public AtlVerkehrsStaerkeStunde(final Data data) {
		instanceData = data;
		_hasState = "atl.verkehrsStärkeStunde".equals(data.getAttributeType().getPid());
	}

	public void doInitialize() {
		new AtlStatus(instanceData.getItem("Status")).doInitialize();
		new AtlGuete(instanceData.getItem("Güte")).doInitialize();
		instanceData.getScaledValue("Wert").setText(AtlVerkehrsStaerkeStunde.ZUSTAND_WERT_NICHT_ERMITTELBAR);
	}

	public IntegerValueRange getRangeOfVerkehrsStaerkeStunde() {
		return ValueRangeHashtable.getInstance(this.instanceData).get("att.verkehrsStärkeStunde");
	}

	public int getWert() {
		return instanceData.getScaledValue("Wert").intValue();
	}

	public void setWert(final int wert) {
		try {
			if (wert < 0) {
				instanceData.getUnscaledValue("Wert").set(wert);
			} else {
				instanceData.getScaledValue("Wert").set(wert);
			}
		} catch (final IllegalArgumentException ex) {
			final long lMaxValue = getRangeOfVerkehrsStaerkeStunde().getMaximum();
			final long lMinValue = getRangeOfVerkehrsStaerkeStunde().getMinimum();
			throw new IllegalArgumentException("Wert für VerkehrsStärkeStunde konnte nicht auf " + wert
					+ " gesetzt werden. (Min/Max:" + lMinValue + "/" + lMaxValue + ")");
		}
	}

	// _Data.getScaledValue("Wert").set(wert);
	// }
	public IntegerValueState getZustandWert() {
		return instanceData.getScaledValue("Wert").getState();
	}

	public void setZustandWert(final IntegerValueState integerValueState) {
		instanceData.getScaledValue("Wert").setState(integerValueState);
	}

	public void setZustandWert(final String zustandsName) {
		instanceData.getScaledValue("Wert").setText(zustandsName);
	}

	public boolean isState() {
		return instanceData.getUnscaledValue("Wert").isState();
	}

	public boolean isNumber() {
		return !isState();
	}

	/**
	 * liefert AtlStatus-Instance zur&uuml;ck, wenn das gekapselte Data vom Typ
	 * atl.verkehrsStaerkeStunde ist und einen Status besitzt (sonst null).
	 *
	 * @return der ermittelte Status
	 */
	public AtlStatus getStatus() {
		if (_hasState) {
			return new AtlStatus(instanceData.getItem("Status"));
		}
		return null;
	}

	/**
	 * liefert true zur&uuml;ck, wenn das gekapselte Data vom Typ
	 * atl.verkehrsStaerkeStunde ist und einen Status besitzt.
	 *
	 * @return boolean
	 */
	public boolean hasState() {
		return _hasState;
	}

	public AtlGuete getGuete() {
		return new AtlGuete(instanceData.getItem("Güte"));
	}

	public Data getData() {
		return instanceData;
	}

	/**
	 * gibt den Wert mit einem Pr&uuml;fkennzeichen zur&uuml;ck.
	 *
	 * <p>
	 * <table align=center border=1 cellspacing=0 cellpadding=3 summary=
	 * "Mögliche Werte">
	 * <tr>
	 * <td align=center>-</td> <td>korrekter regelm&auml;ssiger Wert</td>
	 * <td>&nbsp;</td> </tr> <tr> <td align=center>u</td> <td>fehlender Wert
	 * (Ausfall)</td> <td>wenn Status == nicht ermittelbar, Wert:=0</td> </tr>
	 * <tr> <td align=center>d</td> <td>fehlerhafter Wert</td> <td>wenn
	 * G&uuml;te &lt;&gt;1, Wert wird aus der Attributegruppe
	 * &uuml;bernommen.</td> </tr> </table>
	 *
	 * @return String
	 */
	public String getValueAsBastString() {

		String retValue;
		if (this.isState()) {
			retValue = "0a";
		} else {
			final AtlGuete guete = this.getGuete();
			final StringBuffer sb = new StringBuffer(5);
			sb.append(this.getWert());
			// if (guete.isNumber() && (guete.getIndex()>0.9999999)) {
			if (guete.isNumber()) {
				sb.append('-');
			} else {
				sb.append('d');
			}
			retValue = sb.toString();

		}
		return retValue;
	}

}
