/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

import java.util.Hashtable;

import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.DataModel;

/**
 * kapselt die unterschiedlichen Typen der FG06.
 */
public class FG06Type {

	public static final String ATGSTRING_TUER_KONTAKT = "atg.tlsVltTürKontakt";
	public static final String ATGSTRING_TEMPERATUR_UEBERWACHUNG = "atg.tlsVltTemperaturÜberwachung";
	public static final String ATGSTRING_LICHT = "atg.tlsVltLicht";
	public static final String ATGSTRING_STROM_VERSORGUNG = "atg.tlsVltStromVersorung";
	public static final String ATGSTRING_HEIZUNG = "atg.tlsVltHeizung";
	public static final String ATGSTRING_LUEFTUNG = "atg.tlsVltLüftung";
	public static final String ATGSTRING_UEBERSPANNUNGS_SCHUTZ = "atg.tlsVltÜberspannungsSchutz";
	public static final String ATGSTRING_DIEBSTAHL_VANDALISMUS_SCHUTZ = "atg.tlsVltDiebstahlVandalismusSchutz";
	public static final String ATGSTRING_FERN_UEBWERWACHUNG_SOLAR_ANLAGEN = "atg.tlsVltFernÜberwachungSolarAnlagen";

	public static final short STATUS_OK = 0;
	public static final short STATUS_WARNING = 1;
	public static final short STATUS_ALERT = 2;

	private static Hashtable<String, FG06Type> __MappingTyp = new Hashtable<String, FG06Type>(
			30);

	String _Name;
	String _Abbreviation;
	String _UmsetzungsModul;
	int _Type;
	AttributeGroup _Atg;

	static boolean __NotInitialized = true;

	public FG06Type(final String umsetzungsModul, final String name,
			final String abbreviation, final int type, final String atgPid) {
		_UmsetzungsModul = umsetzungsModul;
		_Name = name;
		_Abbreviation = abbreviation;
		_Type = type;
		final DataModel model = de.bsvrz.dua.bastband.bastband.dvs.funclib.LogosClientDavConnection
				.getInstance().getDataModel();
		try {
			_Atg = model.getAttributeGroup(atgPid);
		} catch (final Exception ex) {
			ex.printStackTrace();
		}
		FG06Type.__MappingTyp.put(_UmsetzungsModul, this);
	}

	public static FG06Type getInstance(final String umsetzungsModul) {
		if (FG06Type.__NotInitialized) {
			FG06Type.init();
		}
		return FG06Type.__MappingTyp.get(umsetzungsModul);
	}

	private synchronized static void init() {

		if (FG06Type.__NotInitialized) {
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ048", "Türkontakt",
					"Tür", 48, FG06Type.ATGSTRING_TUER_KONTAKT);
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ049",
					"Temperaturüberwachung", "Temp", 55,
					FG06Type.ATGSTRING_TEMPERATUR_UEBERWACHUNG);
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ050", "Licht",
					"Licht", 58, FG06Type.ATGSTRING_LICHT);

			new FG06Type("kex.tls.osi7.conversion.Fg006Typ051",
					"Stromversorgung", "Strom", 52,
					FG06Type.ATGSTRING_STROM_VERSORGUNG);
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ052", "Heizung",
					"Heizung", 60, FG06Type.ATGSTRING_HEIZUNG);
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ053", "Lüftung",
					"Lüftung", 56, FG06Type.ATGSTRING_LUEFTUNG);

			new FG06Type("kex.tls.osi7.conversion.Fg006Typ054",
					"Überspannungsschutz", "Spannung", 53,
					FG06Type.ATGSTRING_UEBERSPANNUNGS_SCHUTZ);
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ055",
					"Diebstahl Vandalismus", "DiebVanadal", 49,
					FG06Type.ATGSTRING_DIEBSTAHL_VANDALISMUS_SCHUTZ);
			new FG06Type("kex.tls.osi7.conversion.Fg006Typ056",
					"Fernüberwachung von Solaranlagen", "Solar", 57,
					FG06Type.ATGSTRING_FERN_UEBWERWACHUNG_SOLAR_ANLAGEN);
		}
		FG06Type.__NotInitialized = false;
	}

	public final String getUmsetzungsModul() {
		return _UmsetzungsModul;
	}

	/**
	 * Typ der FG06 (Was wurde gemessen).
	 *
	 * @return Lufttemperatur, Dahrbahnoberfl&auml;chentemperatur, Restsalz usw)
	 */
	public final String getName() {
		return _Name;
	}

	public final String getAbbreviation() {
		return _Abbreviation;
	}

	public final int getType() {
		return _Type;
	}

	public final AttributeGroup getAtg() {
		return _Atg;
	}

	@Override
	public String toString() {
		final StringBuffer sb = new StringBuffer("FG06Type(");
		sb.append(_UmsetzungsModul).append(" - ");
		sb.append(_Name).append(" - ");
		sb.append(_Abbreviation).append(" - ");
		sb.append(_Type).append(" - ");
		sb.append(_Atg.getPid());
		sb.append(")");
		return sb.toString();
	}

}
