/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper;

/**
 * History:
 *
 * $Log: KriBus.java,v $
 * Revision 1.1  2007/07/04 11:44:00  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:26  Trier
 * dvs alles
 *
 * Revision 1.6  2006/04/28 05:48:49  Trier
 * Generics 1.5 verwendet
 *
 * Revision 1.5  2005/11/30 06:58:30  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2005/11/09 14:23:04  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/04/21 09:15:34  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/01/25 09:43:11  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 13:26:02  Trier
 * <Kein Kommentar eingegeben>
 *
 */

import java.util.ArrayList;
import java.util.List;

/**
 * kapselt die Busse zwischen den KRI's und den Steuermodulen.
 * <p>
 * Die Bezeichnung des Busses wird bestimmt, indem der letzte Teil der Pid des
 * zugeh&ouml;rigen AnschlussPunktes interpretiert wird. Enth&auml;lt der letzte
 * Teil der Pid das Wort "Funk" bekommt der Bus die Bezeichnung <i>Funkbus</i>
 * sonst <i>"Bus" || letzterTeil</i>.
 * </p>
 */
public class KriBus {

	private final String _Pid;
	private String _Bezeichnung;
	private final List<SteuerModul> _SteuerModulList = new ArrayList<SteuerModul>();
	private boolean _IsFunkBus;

	/**
	 * gibt die Bezeichnung des Busses zur&uuml;ck, der auf Basis der Pid des
	 * zugeh&ouml;rigen AnschlussPunktes bestimmt wird.
	 *
	 * @return String
	 */
	public String getBezeichnung() {
		return _Bezeichnung;
	}

	public boolean isFunkBus() {
		return _IsFunkBus;
	}

	/**
	 * gibt die SteuerModule zur&uuml;ck oder null, wenn keine angeschlossen
	 * sind.
	 *
	 * @return SteuerModule.
	 */
	public SteuerModul[] getSteuerModule() {
		if ((_SteuerModulList != null) && (_SteuerModulList.size() > 0)) {
			return _SteuerModulList.toArray(new SteuerModul[_SteuerModulList
			                                                .size()]);
		} else {
			return null;
		}
	}

	/**
	 * wird von der Klasse {@link KRI} benutzt, um die an den Bus
	 * angeschlossenen Steuermodule festzulegen.
	 *
	 * @param sm
	 *            SteuerModul
	 */
	public void addSteuerModul(final SteuerModul sm) {
		_SteuerModulList.add(sm);
		sm.setBus(this);
	}

	/**
	 * erzeugt eine Instance und bestimmt die Bezeichnung des Busses.
	 *
	 * @param pid
	 *            Pid des AnschlussPunktes am Kri
	 */
	protected KriBus(final String pid) {
		_Pid = pid;
		final String[] strings = pid.split("\\.");
		if ((strings != null) && (strings.length > 0)) {
			if (strings[strings.length - 1].toLowerCase().indexOf("funk") > -1) {
				_Bezeichnung = "Funkbus";
				_IsFunkBus = true;
			} else {
				_Bezeichnung = "BUS " + strings[strings.length - 1];
				_IsFunkBus = false;
			}
		} else {
			_Bezeichnung = pid;
			_IsFunkBus = false;
		}
	}

	/**
	 * gibt die Bezeichnung des Busses zur&uuml;ck.
	 *
	 * @return String
	 */
	@Override
	public String toString() {
		return _Bezeichnung;
	}

	public String getPID() {
		return _Pid;
	}

}
