/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtgVerkehrsdatenLangZeitIntervall;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlGeschwindigkeit;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlGuete;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtlVerkehrsStaerkeStunde;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;

/**
 * History:
 *
 * $Log: DataChannelBAStV1_X.java,v $
 * Revision 1.6  2007/08/22 09:11:17  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2007/07/26 12:49:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/25 13:02:06  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.5  2007/01/16 09:56:50  Trier
 * Erweiterung BastImport/Export
 * insbesondere die Unterstützung  unterschiedl. Aspekte
 *
 * Revision 1.4  2006/11/13 09:25:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2005/02/28 14:40:47  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/02/24 07:01:01  Trier
 * Variante ohne Protokoll
 *
 * Revision 1.1  2005/02/23 09:15:20  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/02/18 10:09:51  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/31 15:16:10  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 14:40:07  Trier
 * <Kein Kommentar eingegeben>
 *
 */

/**
 * <p>
 * Channel zur Ausgabe von Daten in das BASt-Bestandsbandformat
 * </p>
 *
 * <p>
 * Es ist zubeachten, dass die Anzahl der Geschwindigkeitsklassen nicht ohne
 * weiteres zu bestimmen ist. Es wird davon ausgegangen, dass es eine Festlegung
 * &uuml;ber die Geschwindigkeitsklassifizierung gibt, die sich nicht
 * &auml;ndert.
 * </p>
 *
 * <p>
 * Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt
 * asp.messWertErsetzung ausgegeben.
 * </p>
 * <p>
 * Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen
 * ist der folgenen Tabelle zu entnehmen.
 * </p>
 * <table border=1 align=center cellpadding=2 cellspacing=0 summary="Werte">
 * <tr>
 * <th align=left>Pr&uuml;fkennzeichen</th> <th align=left>Bemerkung</th> </tr>
 * <tr> <td>&nbsp;</td> <td>korrekte, regelmäßige Werte</td> </tr> <tr>
 * <td>'</td> <td>alle anderen Werte</td> </tr> </table>
 *
 * <p> Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr
 * ausgegeben. </p>
 *
 * <p> siehe auch:
 * {@link de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.AbstractDataChannelBASt
 * AbstractDataChannelBASt} </p>
 *
 */
abstract public class DataChannelBAStV1_X extends AbstractDataChannelBASt {

	private static final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger LOGGER = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
			.getLoggerFactory().getLogger();

	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r die den G-Satz
	 */
	public final static int G_SATZ = 10;
	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r die den K-Satz
	 */
	public final static int K_SATZ = 11;
	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r die den V-Satz
	 */
	public final static int V_SATZ = 12;
	/**
	 * Konstante zum Erzeugen ein Channels f&uuml;r die den M-Satz
	 */
	public final static int M_SATZ = 13;

	private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyDDD");
	private static DecimalFormat HOUR_FORMAT = new DecimalFormat("00");
	protected LangZeitZaehlStelle _ZaehlStelle;
	private ValueExtractor _ValueExtractor;

	protected Calendar chCalendar;

	protected boolean hasNoData = true;

	protected int countOfGeschwKlassen = 0;

	private boolean exportGSatz = true;
	private boolean exportKSatz = true;
	private boolean exportVSatz = true;
	private boolean exportMSatz = true;

	/**
	 * gibt an mit welcher Zahl die Stundenz&auml;lung beginnt (0 oder 1; 1 ist
	 * Standard).
	 */
	private int _FirstHour = 1;

	/**
	 * gibt an, ob zum Abschlu&szlig; einer Zeile ein Semikolon gesetzt werden
	 * soll.
	 */
	private boolean endLineWithSemikolon = false;

	/**
	 * f&uuml;r eine Z&auml;hlstelle konstanter Anfang jeder Zeile.
	 */
	private String lineHeader;

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link AbstractDataChannelBASt#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 */
	public DataChannelBAStV1_X() {
	}

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link AbstractDataChannelBASt#setZaehlstelle(LangZeitZaehlStelle zaehlstelle)
	 * setZaehlstelle()} initialisiert werden muss zur Ausgabe der angegebenen
	 * SatzArt.
	 */
	public DataChannelBAStV1_X(final int satzArt) {
		exportGSatz = false;
		exportKSatz = false;
		exportVSatz = false;
		exportMSatz = false;

		switch (satzArt) {
		case G_SATZ: {
			exportGSatz = true;
			exportKSatz = false;
			exportVSatz = false;
			exportMSatz = false;
			break;
		}
		case K_SATZ: {
			exportGSatz = false;
			exportKSatz = true;
			exportVSatz = false;
			exportMSatz = false;
			break;
		}
		case V_SATZ: {
			exportGSatz = false;
			exportKSatz = false;
			exportVSatz = true;
			exportMSatz = false;
			break;
		}
		case M_SATZ: {
			exportGSatz = false;
			exportKSatz = false;
			exportVSatz = false;
			exportMSatz = true;
			break;
		}
		}
	}

	/**
	 * setzt die Anzahl der Geschwindigkeitsklassen.
	 *
	 * @param count
	 *            int
	 */
	public void setCountOfGeschwKlassen(final int count) {
		countOfGeschwKlassen = count;
	}

	/**
	 * gint die Anzahl der Geschwindigkeitsklassen zur&uuml;ck.
	 *
	 * @return int
	 */
	public int getCountOfGeschwKlassen() {
		return countOfGeschwKlassen;
	}

	/**
	 * erzeugt einen DataChannel, der vor dem Aufruf von
	 * {@link de.bsvrz.dua.bastband.bastband.dvs.funclib.ResultDataChannel#processRecord(Dataset[], Calendar)}
	 * mit {@link #setZaehlstelle(LangZeitZaehlStelle)} initialisiert werden
	 * muss.
	 *
	 * <p>
	 * Ist die Datei vorhanden wird diese &uuml;berschrieben.
	 * </p>
	 *
	 * @param fileName
	 *            Dateiname mit vollst&auml;ndigem Pfad
	 */
	public DataChannelBAStV1_X(final String fileName) throws ChannelException {
		super(fileName);
	}

	/**
	 * legt fest, ob der M-Satz geschrieben werden soll
	 * (standardm&auml;&szlig;ig werden alle geschrieben).
	 *
	 * @param exportMSatz
	 *            boolean
	 */
	public void setExportMSatz(final boolean exportMSatz) {
		this.exportMSatz = exportMSatz;
	}

	/**
	 * legt fest, ob der V-Satz geschrieben werden soll
	 * (standardm&auml;&szlig;ig werden alle geschrieben).
	 *
	 * @param exportVSatz
	 *            boolean
	 */
	public void setExportVSatz(final boolean exportVSatz) {
		this.exportVSatz = exportVSatz;
	}

	/**
	 * legt fest, ob der G-Satz geschrieben werden soll
	 * (standardm&auml;&szlig;ig werden alle geschrieben).
	 *
	 * @param exportGSatz
	 *            boolean
	 */
	public void setExportGSatz(final boolean exportGSatz) {
		this.exportGSatz = exportGSatz;
	}

	/**
	 * legt fest, ob der K-Satz geschrieben werden soll
	 * (standardm&auml;&szlig;ig werden alle geschrieben).
	 *
	 * @param exportKSatz
	 *            boolean
	 */
	public void setExportKSatz(final boolean exportKSatz) {
		this.exportKSatz = exportKSatz;
	}

	/**
	 * setzt die Ziffer, mit der die Stuhndenz&auml;lung beginnt ( 0 oder 1; 1
	 * == Standard).
	 *
	 * @param firstHour
	 *            0 oder 1
	 */
	public void setFirstHourOfDay(final int firstHour) {
		_FirstHour = firstHour;
	}

	/**
	 * bestimmt, ob am Ende einer Zeile ein Semikolon gesetzt werden soll
	 * (Standard == nein).
	 *
	 * @param endLineWithSemikolon
	 *            true, wenn am Zeilenende ein Semikolon gesetzt werden soll.
	 */
	public void setEndLineWithSemikolon(final boolean endLineWithSemikolon) {
		this.endLineWithSemikolon = endLineWithSemikolon;
	}

	/**
	 * erzeugt die BASt-Format-Daten und spooled diese.
	 *
	 * <p>
	 * vor dem ersten Aufruf muss, der Channel vollst&auml; initialisiert sein.
	 * </p>
	 *
	 * @param rsArrayAtTime
	 *            ResultData[]
	 * @param calendar
	 *            Intervallbeginn der Datens&auml;tze
	 * @throws ChannelException
	 */
	@Override
	public void processRecord(final Dataset[] rsArrayAtTime, final Calendar calendar) throws ChannelException {

		final int actHour = calendar.get(Calendar.HOUR);
		chCalendar = (Calendar) calendar.clone();

		final String sZeitString = DataChannelBAStV1_X.DATE_FORMAT.format(chCalendar.getTime())
				+ DataChannelBAStV1_X.HOUR_FORMAT.format(chCalendar.get(Calendar.HOUR_OF_DAY) + _FirstHour);

		AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = null;
		if (rsArrayAtTime != null) {
			int iAnzFSOhneDaten = 0;
			atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];
			for (int i = 0; i < _CountOfFS; i++) {
				if ((rsArrayAtTime[i] != null) && (rsArrayAtTime[i].getData() != null)) {
					atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArrayAtTime[i].getData());
				} else {
					iAnzFSOhneDaten++;
				}
			}
		}
		try {
			if ((rsArrayAtTime != null) && (rsArrayAtTime.length > 0)) {
				if (exportKSatz) {
					final StringBuffer sbLine = new StringBuffer(180);
					sbLine.append(lineHeader).append("K1").append(sZeitString);
					// StringBuffer s =
					// _ValueExtractor.extractDataKSatz(atgLZArray);
					sbLine.append(_ValueExtractor.extractDataKSatz(atgLZArray));
					if (endLineWithSemikolon) {
						sbLine.append(';');
					}
					spool(sbLine);
				}
				if (exportGSatz) {
					final StringBuffer sbLine = new StringBuffer(180);
					sbLine.append(lineHeader).append("G1").append(sZeitString);
					sbLine.append(extractDataGSatz(atgLZArray));
					if (endLineWithSemikolon) {
						sbLine.append(';');
					}
					spool(sbLine);
				}
				if (exportVSatz) {
					final StringBuffer sbLine = new StringBuffer(180);
					sbLine.append(lineHeader).append("V1").append(sZeitString);
					sbLine.append(extractDataVSatz(atgLZArray));
					if (endLineWithSemikolon) {
						sbLine.append(';');
					}
					spool(sbLine);
				}
				if (exportMSatz) {
					final StringBuffer sbLine = new StringBuffer(180);
					sbLine.append(lineHeader).append("M1").append(sZeitString);
					sbLine.append(extractDataMSatz(atgLZArray));
					if (endLineWithSemikolon) {
						sbLine.append(';');
					}
					spool(sbLine);
				}
			} else {
				if (rsArrayAtTime == null) {
					DataChannelBAStV1_X.LOGGER.finer("rsArrayAtTime is null");
				} else {
					DataChannelBAStV1_X.LOGGER.finer("rsArrayAtTime.size: %s", rsArrayAtTime.length);
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();

		}
		_LastHour = actHour;
	}

	/**
	 * erzeugt den f&uuml;r eine Z&auml;hlstelle konstanten Zeilenanfang. Der
	 * Anfang jeder Zeile besteht aus TK-BlattNr, Z&auml;hlstellenNr.,
	 * L&auml;nderkennung.
	 */
	abstract protected String calculateLineHeader();

	@Override
	public void setZaehlstelle(final LangZeitZaehlStelle zaehlstelle) throws ChannelException {

		_ZaehlStelle = zaehlstelle;
		final String sKlassifizierungsgruppeBAST = _ZaehlStelle.getKlassifizierungsgruppeBAST();
		setKlassifizierungsgruppeBAST(sKlassifizierungsgruppeBAST);
		final int fsCountInRichtg = zaehlstelle.getInAuswertequerschnitt().getMessQuerschnitt()
				.getFahrstreifen().length;
		final int fsCountGegenRichtg = zaehlstelle.getGegenAuswertequerschnitt().getMessQuerschnitt()
				.getFahrstreifen().length;
		_CountOfFS = fsCountInRichtg + fsCountGegenRichtg;

		lineHeader = this.calculateLineHeader();
	}

	protected ValueExtractor getValueExtractor8plus1() {
		return new ValueExtractor8plus1();
	}

	/**
	 * setzt die KlassifizierungsgruppeBASt ("8 plus 1" | "5 plus 1" | "2" | "1"
	 * ).
	 * <p>
	 * muss nach {@link #setZaehlstelle(LangZeitZaehlStelle)} erfolgen,
	 * ansonsten wird die Klassifizierungsgruppe aus der Konfiguration
	 * übernommen.
	 *
	 * @param sKlassifizierungsgruppeBAST
	 *            String
	 * @throws ChannelException
	 */
	public void setKlassifizierungsgruppeBAST(final String sKlassifizierungsgruppeBAST) throws ChannelException {
		if ("8 plus 1".equals(sKlassifizierungsgruppeBAST)) {
			_ValueExtractor = getValueExtractor8plus1();
			DataChannelBAStV1_X.LOGGER.finer("ValueExtractor8plus1");
		} else {
			if ("5 plus 1".equals(sKlassifizierungsgruppeBAST)) {
				_ValueExtractor = new ValueExtractor5plus1();
				DataChannelBAStV1_X.LOGGER.finer("ValueExtractor5plus1");
			} else {
				if ("2".equals(sKlassifizierungsgruppeBAST)) {
					_ValueExtractor = new ValueExtractor2();
					DataChannelBAStV1_X.LOGGER.finer("ValueExtractor2");
				} else {
					if ("1".equals(sKlassifizierungsgruppeBAST)) {
						_ValueExtractor = new ValueExtractor1();
						DataChannelBAStV1_X.LOGGER.finer("ValueExtractor1");
					} else {
						throw new ChannelException(
								"Klassifizierungsgruppe \"" + sKlassifizierungsgruppeBAST + "\" ist nicht gültig");
					}
				}
			}
		}
	}

	@Override
	protected String constructFilename(final LangZeitZaehlStelle zaehlStelle, final String path, final int year,
			final int startDayOfYear, final int endDayOfYear) {
		final String bastLzzNummer = _ZaehlStelle.getBastNummerAsString();
		String sYear = String.valueOf(year % 100);
		if (sYear.length() == 1) {
			sYear = '0' + sYear;
		}
		final StringBuffer sb = new StringBuffer(path);
		sb.append(bastLzzNummer).append('_');
		sb.append('_').append(AbstractDataChannelBASt.THREE_DIGIT_FORMAT.format(startDayOfYear));
		sb.append('_').append(AbstractDataChannelBASt.THREE_DIGIT_FORMAT.format(endDayOfYear));
		final int iSatzArt = getSatzType();
		if (iSatzArt == 0) {
			sb.append(".txt");
		} else {
			switch (iSatzArt) {
			case G_SATZ: {
				sb.append('G');
				break;
			}
			case K_SATZ: {
				sb.append('K');
				break;
			}
			case V_SATZ: {
				sb.append('V');
				break;
			}
			case M_SATZ: {
				sb.append('M');
				break;
			}
			}
		}
		DataChannelBAStV1_X.LOGGER.finer("FileName: %s", sb);
		return sb.toString();
	}

	@Override
	protected String constructFilename(final LangZeitZaehlStelle zaehlStelle, final String path, final int year,
			final int monthOrQuartal, final boolean isQuartal) {
		final StringBuffer sb = new StringBuffer(path);
		final String bastLzzNummer = _ZaehlStelle.getBastNummerAsString();
		sb.append(bastLzzNummer);
		sb.append('_').append(AbstractDataChannelBASt.TWO_DIGIT_FORMAT.format(year % 100));

		if (isQuartal) {
			sb.append("Q.").append(monthOrQuartal);
		} else {
			sb.append(monthOrQuartal + 1);
		}

		final int iSatzArt = getSatzType();
		if (iSatzArt == 0) {
			sb.append(".txt");
		} else {
			switch (iSatzArt) {
			case G_SATZ: {
				sb.append('G');
				break;
			}
			case K_SATZ: {
				sb.append('K');
				break;
			}
			case V_SATZ: {
				sb.append('V');
				break;
			}
			case M_SATZ: {
				sb.append('M');
				break;
			}
			}
		}
		DataChannelBAStV1_X.LOGGER.finer("FileName: %s", sb);
		return sb.toString();
	}

	private int getSatzType() {
		int iSatzType = 0;
		if (exportGSatz) {
			iSatzType = DataChannelBAStV1_X.G_SATZ;
		}
		if (exportKSatz) {
			iSatzType = iSatzType + DataChannelBAStV1_X.K_SATZ;
		}
		if (exportVSatz) {
			iSatzType = iSatzType + DataChannelBAStV1_X.V_SATZ;
		}
		if (exportMSatz) {
			iSatzType = iSatzType + DataChannelBAStV1_X.M_SATZ;
		}
		if (iSatzType < 20) {
			return iSatzType;
		} else {
			return 0;
		}
	}

	@Override
	protected void writeHeader(final PrintWriter writer) {
	}

	private final static String format(final String s) {
		String retValue = s;
		switch (s.length()) {
		case 1:
			retValue = "    " + s;
			break;
		case 2:
			retValue = "   " + s;
			break;
		case 3:
			retValue = "  " + s;
			break;
		case 4:
			retValue = " " + s;
			break;
		}
		return retValue;
	}

	protected StringBuffer extractDataGSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {

		final StringBuffer sb = new StringBuffer();
		// Werte: vPkwÄ, v85PkwÄ, sPkwÄ, vLkwÄ, sLkwÄ für alle Fahrstreifen
		for (int i = 0; i < _CountOfFS; i++) {
			if ((atgLZArray == null) || (atgLZArray[i] == null)) {
				sb.append("'    '    '    '    '    ");
			} else {
				sb.append(formatBastST(atgLZArray[i].getvPkwAE()));
				sb.append(formatBastST(atgLZArray[i].getv85PkwAE()));
				sb.append(formatBastST(atgLZArray[i].getsPkwAE()));
				sb.append(formatBastST(atgLZArray[i].getvLkwAE()));
				sb.append(formatBastST(atgLZArray[i].getsLkwAE()));
			}
		}
		return sb;
	}

	protected StringBuffer extractDataMSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray)
			throws ChannelException {
		StringBuffer retValue = null;
		try {
			// Bestimmen der Anzahl der Klassen
			if (countOfGeschwKlassen == 0) {
				for (int i = 0; ((i < _CountOfFS) && (countOfGeschwKlassen == 0)); i++) {
					final AtlVerkehrsStaerkeStunde[] atlArray = atgLZArray[i].getqPkwAEGeschwKlassen();
					if ((atlArray != null) && (atlArray.length > 0)) {
						_CountOfFS = atlArray.length;
						retValue = extractDataMSatz(atgLZArray);
					}
				}
			} else {
				retValue = new StringBuffer();
				for (int i = 0; i < _CountOfFS; i++) {
					AtlVerkehrsStaerkeStunde[] atlPkwArray = null;
					AtlVerkehrsStaerkeStunde[] atlLkwArray = null;
					if (atgLZArray[i] != null) {
						atlPkwArray = atgLZArray[i].getqPkwAEGeschwKlassen();
						atlLkwArray = atgLZArray[i].getqLkwAEGeschwKlassen();
					}
					if ((atlPkwArray != null) && (atlPkwArray.length == countOfGeschwKlassen)) {
						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append(formatBastMV_Test(atlPkwArray[kl]));
						}
					} else {

						if ((atlPkwArray != null) && (atlPkwArray.length != countOfGeschwKlassen)) {

							logWarning(chCalendar.getTime(),
									"Einteilung der Geschwindigkeitsklassen für Pkw des " + i
									+ ". Fahrstreifen stimmt nicht (Ist: " + atlPkwArray.length + "  Soll: "
									+ countOfGeschwKlassen + ").");

						}

						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append("'    ");
						}
					}
					if ((atlLkwArray != null) && (atlLkwArray.length == countOfGeschwKlassen)) {
						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append(formatBastMV_Test(atlLkwArray[kl]));
						}
					} else {

						if ((atlLkwArray != null) && (atlLkwArray.length != countOfGeschwKlassen)) {
							logWarning(chCalendar.getTime(),
									"Einteilung der Geschwindigkeitsklassen für Lkw des " + i
									+ ". Fahrstreifen stimmt nicht (Ist: " + atlLkwArray.length + "  Soll: "
									+ countOfGeschwKlassen + ").");
						}

						for (int kl = 0; kl < countOfGeschwKlassen; kl++) {
							retValue.append("'    ");
						}
					}
				}
			}
		} catch (final Exception e) {
			e.printStackTrace();
			throw new ChannelException("Fehler beim Extrahieren der Daten für dem M-Satz, Zeile wurde geschrieben.", e);
		}
		return retValue;
	}

	protected StringBuffer extractDataVSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {
		final StringBuffer sb = new StringBuffer();
		// Werte: nkKfz, Krad, Pkw, LfW, PkwA, Lkw, LkwA, SattelKfz, Busse für
		// alle Fahrstreifen
		for (int i = 0; i < _CountOfFS; i++) {
			if ((atgLZArray == null) || (atgLZArray[i] == null)) {
				sb.append("'    '    ");
			} else {
				sb.append(formatBastMV_Test(atgLZArray[i].getqKfz()));
				sb.append(formatBastMV_Test(atgLZArray[i].getqLkwAE()));
				// System.out.println("V-Satz="+sb);
			}
		}
		return sb;
	}

	protected interface ValueExtractor {
		StringBuffer extractDataKSatz(AtgVerkehrsdatenLangZeitIntervall[] rsArray);
	}

	/**
	 * extrahiert die Werte aus den ResultData und baut einen String mit den
	 * Werten f&uuml;r die Klassifizierungsgruppe 8plus1.
	 */
	private class ValueExtractor8plus1 implements ValueExtractor {

		@Override
		public StringBuffer extractDataKSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {

			final StringBuffer sb = new StringBuffer();
			// Werte: nkKfz, Krad, Pkw, LfW, PkwA, Lkw, LkwA, SattelKfz, Busse
			// für alle Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if ((atgLZArray == null) || (atgLZArray[i] == null)) {
					sb.append("'    '    '    '    '    '    '    '    '    ");
				} else {
					sb.append(formatBastMV_Test(atgLZArray[i].getqKfzNk()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqKrad()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqPkw()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqLfw()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqPkwA()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqLkw()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqLkwA()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqSattelKfz()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqBus()));
				}
			}
			return sb;
		}

	}

	/**
	 * extrahiert die Werte aus den ResultData und baut einen String mit den
	 * Werten f&uuml;r die Klassifizierungsgruppe 5plus1.
	 */
	private class ValueExtractor5plus1 implements ValueExtractor {

		@Override
		public StringBuffer extractDataKSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {

			final StringBuffer sb = new StringBuffer();
			// Werte: PkwG, PkwA, Lkw, LkwK, Bus, nkKfz für alle Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if ((atgLZArray == null) || (atgLZArray[i] == null)) {
					sb.append("'    '    '    '    '    '    ");
				} else {
					sb.append(formatBastMV_Test(atgLZArray[i].getqPkwG()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqPkwA()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqLkw()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqLkwK()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqBus()));
					sb.append(formatBastMV_Test(atgLZArray[i].getqKfzNk()));
				}
			}
			return sb;
		}

	}

	/**
	 * erzeugt einen String mit den Werten f&uuml;r die Klassifizierungsgruppe
	 * PkwAehnliche und LkwAehnliche (2 Gruppe).
	 */
	private class ValueExtractor2 implements ValueExtractor {

		@Override
		public StringBuffer extractDataKSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {
			return null;
		}
	}

	/**
	 * erzeugt einen String mit den Werten f&uuml;r die Klassifizierungsgruppe
	 * Kfz (1 Gruppe).
	 */
	private class ValueExtractor1 implements ValueExtractor {

		@Override
		public StringBuffer extractDataKSatz(final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray) {
			return null;
		}
	}

	private String formatBastMV_Test(final AtlVerkehrsStaerkeStunde atl) {
		String retValue;
		if (atl.isState()) {
			retValue = "'    ";
		} else {
			final AtlGuete guete = atl.getGuete();
			// if (guete.isNumber() && (guete.getIndex()>0.9999999)) {
			if (guete.isNumber()) {
				final StringBuffer sb = new StringBuffer(5);
				sb.append(atl.getWert());
				while (sb.length() < 4) {
					sb.insert(0, '0');
				}
				sb.insert(0, ' ');
				retValue = sb.toString();
			} else {
				retValue = "'    ";
			}
		}
		return retValue;
	}

	private String formatBastST(final AtlGeschwindigkeit atl) {
		String retValue;
		if (atl.isState()) {
			retValue = "'    ";
		} else {
			final AtlGuete guete = atl.getGuete();
			// if (guete.isNumber() && (guete.getIndex()>0.9999999)) {
			if (guete.isNumber()) {
				final StringBuffer sb = new StringBuffer(5);
				sb.append(atl.getWert() * 10).append('0');
				while (sb.length() < 4) {
					sb.insert(0, '0');
				}
				sb.insert(0, ' ');
				retValue = sb.toString();
			} else {
				retValue = "'    ";
			}
		}
		return retValue;
	}

}
