/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

/**
 * History:
 *
 * $Log: DataChannelBAStV2_0.java,v $
 * Revision 1.6  2007/08/22 07:48:45  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.5  2007/07/26 12:49:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/25 13:02:06  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.3  2006/11/13 09:25:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/09/22 06:56:26  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 14:40:05  Trier
 * <Kein Kommentar eingegeben>
 *
 */
import java.io.PrintWriter;
import java.text.DecimalFormat;
import java.text.SimpleDateFormat;
import java.util.Calendar;

import de.bsvrz.dav.daf.main.Dataset;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.datawrapper.AtgVerkehrsdatenLangZeitIntervall;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.AuswerteQuerschnitt;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;
import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.Strasse;

/**
 * <p>
 * Channel zur Ausgabe der Verkehrsmengendaten in das BASt-Bestandsbandformat
 * Version 2004.
 * </p>
 *
 * <p>
 * Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt
 * asp.messWertErsetzung ausgegeben.
 * </p>
 * <p>
 * Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen
 * ist der folgenen Tabelle zu entnehmen.
 * </p>
 * <table border=1 align=center cellpadding=2 cellspacing=0 summary="Werte">
 * <tr>
 * <th align=left>Pr&uuml;fkennzeichen</th> <th align=left>Beschreibung lt.
 * BAST</th> <th align=left>Bemerkung</th> </tr> <tr> <td>-</td> <td>korrekte,
 * regelmäßige Werte</td> <td>&nbsp;</td> </tr> <tr> <td>u</td> <td>korrekte,
 * aber unregelmäßige Werte (z.B. wg. Großverantstaltung)</td> <td>wird nicht
 * unterst&uuml;tzt</td> </tr> <tr> <td>a</td> <td>fehlende Werte (Ausfall)</td>
 * <td>wenn Status == "nicht ermittelbar"</td>
 * </tr>
 * <tr>
 * <td>d</td>
 * <td>fehlerhafte Werte</td>
 * <td>wenn G&uuml;te &lt;&gt; 1</td>
 * </tr>
 * <tr>
 * <td>s</td>
 * <td>Sch&auml;tzwert wegen fehlender Werte</td>
 * <td>wird nicht unterst&uuml;tzt</td>
 * </tr>
 * <tr>
 * <td>k</td>
 * <td>Sch&auml;tzwert wegen fehlerhafter Werte</td>
 * <td>wird nicht unterst&uuml;tzt</td>
 * </tr>
 * <tr>
 * <td>z</td>
 * <td>Zeitumstellung (auf MESZ bzw. auf MEZ)</td>
 * <td>&nbsp;</td>
 * </tr>
 * </table>
 *
 * <p>
 * Die fehlende Stunde bei der Umstellung von der MEZ zur MESZ wird mit 0-Werten
 * und Pr&uuml;kennzeichen z (Zeitumstellung) und der Statuskennung i
 * ausgegeben.
 *
 * Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr ausgegeben.
 *
 *
 * <p>
 * siehe auch:
 * {@link de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.AbstractDataChannelBASt
 * AbstractDataChannelBASt}
 * </p>
 */
public class DataChannelBAStV20 extends AbstractDataChannelBASt {

	private static final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger LOGGER = de.bsvrz.dua.bastband.bastband.funclib.logger.LoggerFactory
			.getLoggerFactory().getLogger();

	// private static final DecimalFormat __TWO_DIGIT_FORMAT = new
	// DecimalFormat("00");

	// 1 2 3 4 5
	// 12345678901234567890123456789012345678901234567890123
	// 3 Zeile der BastDatei
	private static String HEADER_ROW03_8plus1 = "S02 09 KFZ SV  Mot Pkw LfW PmA Bus LoA LmA Sat Son;";
	private static String HEADER_ROW03_5plus1 = "S02 06 KFZ SV  PLZ PmA Bus LoA Lzg Son;";
	private static String HEADER_ROW03_2 = "S02 00 KFZ Lkw;";
	private static String HEADER_ROW03_1 = "S01 00 KFZ;";

	// private static SimpleDateFormat DATE_FORMAT = new
	// SimpleDateFormat("yyMMdd HH:mm");
	private static SimpleDateFormat DATE_FORMAT = new SimpleDateFormat("yyMMdd");
	private static DecimalFormat HOUR_FORMAT = new DecimalFormat("00");

	private String _ThirdLine;
	private ValueExtractor _ValueExtractor;

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link AbstractDataChannelBASt#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 */
	public DataChannelBAStV20() {
	}

	/**
	 * erzeugt einen DataChannel, der vor dem Aufruf von
	 * {@link DataChannelBAStV20#processRecord(Dataset[], Calendar)} mit
	 * {@link #setZaehlstelle(LangZeitZaehlStelle)} initialisiert werden muss.
	 *
	 * <p>
	 * Ist die Datei vorhanden wird diese &uuml;berschrieben.
	 * </p>
	 *
	 * @param fileName
	 *            Dateiname mit vollst&auml;ndigem Pfad
	 */
	public DataChannelBAStV20(final String fileName) {
		super(fileName);
	}

	/**
	 * erzeugt die BASt-Format-Daten und spooled diese.
	 *
	 * <p>
	 * vor dem ersten Aufruf muss, der Channel vollst&auml; initialisiert sein.
	 * </p>
	 *
	 * @param rsArrayAtTime
	 *            ResultData[]
	 * @param calendar
	 *            Intervallbeginn der Datens&auml;tze
	 * @throws ChannelException
	 */
	@Override
	public void processRecord(final Dataset[] rsArrayAtTime, final Calendar calendar) throws ChannelException {
		final int actHour = calendar.get(Calendar.HOUR);
		// Prüfen, ob Umstellung MEZ auf MESZ (Sommerzeit), dann gibt es für den
		// letzten
		// Sonntag im März 2 Uhr keine Daten.
		// Eine Zeile muss eingefügt werden.
		if ((_LastHour == 1) && (actHour == 3)) {
			final StringBuffer sbLine = new StringBuffer(180);
			sbLine.append(DataChannelBAStV20.DATE_FORMAT.format(calendar.getTime())).append(' ');
			sbLine.append(DataChannelBAStV20.HOUR_FORMAT.format(calendar.get(Calendar.HOUR_OF_DAY) + 1)).append(":00");
			sbLine.setCharAt(6, 'i');
			// Uhrzeit auf 2 setzen
			sbLine.setCharAt(8, '3');
			sbLine.append(_ValueExtractor.extractData(new ResultData[_CountOfFS])).append(';');
			this.spool(sbLine);
		}
		if ((rsArrayAtTime != null) && (rsArrayAtTime.length > 0)) {
			final StringBuffer sbLine = new StringBuffer(180);
			// sbLine.append(DATE_FORMAT.format(calendar.getTime()));

			sbLine.append(DataChannelBAStV20.DATE_FORMAT.format(calendar.getTime())).append(' ');
			sbLine.append(DataChannelBAStV20.HOUR_FORMAT.format(calendar.get(Calendar.HOUR_OF_DAY) + 1)).append(":00");

			// Die 2. Stunde am letzten Sonntag im Oktober gibt es zweimal
			// die erste schmeissen wir weg.
			// Da nur in diesem Fall eine Stunde doppelt sein kann
			// prüfen wir nur _LastHour == actHour
			if (_LastHour == actHour) {
				this.deleteLastLine();
			}
			sbLine.append(_ValueExtractor.extractData(rsArrayAtTime));
			DataChannelBAStV20.LOGGER.finer("AbstractDataChannelBASt.hasData(rsArrayAtTime)=%s",
					AbstractDataChannelBASt.hasData(rsArrayAtTime));
			if (!AbstractDataChannelBASt.hasData(rsArrayAtTime)) {
				sbLine.setCharAt(6, 'i');
			}
			this.spool(sbLine);
		} else {
			if (rsArrayAtTime == null) {
				DataChannelBAStV20.LOGGER.finer("rsArrayAtTime is null");
			} else {
				DataChannelBAStV20.LOGGER.finer("rsArrayAtTime.size: %s", rsArrayAtTime.length);
			}
		}
		_LastHour = actHour;
	}

	@Override
	public void setZaehlstelle(final LangZeitZaehlStelle zaehlstelle) throws ChannelException {

		_ZaehlStelle = zaehlstelle;
		final String sKlassifizierungsgruppeBAST = _ZaehlStelle.getKlassifizierungsgruppeBAST();
		if ("8 plus 1".equals(sKlassifizierungsgruppeBAST)) {
			_ThirdLine = DataChannelBAStV20.HEADER_ROW03_8plus1;
			_ValueExtractor = new ValueExtractor8plus1();
		} else {
			if ("5 plus 1".equals(sKlassifizierungsgruppeBAST)) {
				_ThirdLine = DataChannelBAStV20.HEADER_ROW03_5plus1;
				_ValueExtractor = new ValueExtractor5plus1();
			} else {
				if ("2".equals(sKlassifizierungsgruppeBAST)) {
					_ThirdLine = DataChannelBAStV20.HEADER_ROW03_2;
					_ValueExtractor = new ValueExtractor2();
				} else {
					if ("1".equals(sKlassifizierungsgruppeBAST)) {
						_ThirdLine = DataChannelBAStV20.HEADER_ROW03_1;
						_ValueExtractor = new ValueExtractor1();
					} else {
						throw new ChannelException("Klassifizierungsgruppe der LZZ nicht gesetzt");
					}
				}
			}
		}
		final int fsCountInRichtg = zaehlstelle.getInAuswertequerschnitt().getMessQuerschnitt()
				.getFahrstreifen().length;
		final int fsCountGegenRichtg = zaehlstelle.getGegenAuswertequerschnitt().getMessQuerschnitt()
				.getFahrstreifen().length;
		_CountOfFS = fsCountInRichtg + fsCountGegenRichtg;
	}

	@Override
	protected String constructFilename(final LangZeitZaehlStelle zaehlStelle, final String path, final int year,
			final int startDayOfYear, final int endDayOfYear) {
		final String bastLzzNummer = _ZaehlStelle.getBastNummerAsString();
		String sYear = String.valueOf(year % 100);
		if (sYear.length() == 1) {
			sYear = '0' + sYear;
		}
		final StringBuffer sb = new StringBuffer(path);
		sb.append(bastLzzNummer).append('_');
		sb.append('_').append(AbstractDataChannelBASt.THREE_DIGIT_FORMAT.format(startDayOfYear));
		sb.append('_').append(AbstractDataChannelBASt.THREE_DIGIT_FORMAT.format(endDayOfYear));
		sb.append(".txt");
		DataChannelBAStV20.LOGGER.finer("FileName: %s", sb);
		return sb.toString();
	}

	@Override
	protected String constructFilename(final LangZeitZaehlStelle zaehlStelle, final String path, final int year,
			final int monthOrQuartal, final boolean isQuartal) {
		final String bundeslandKuerzel = _ZaehlStelle.getBundesland().getAbbreviation();
		final String bastLzzNummer = _ZaehlStelle.getBastNummerAsString();
		final StringBuffer sb = new StringBuffer(path);
		sb.append(bundeslandKuerzel);
		sb.append(bastLzzNummer);
		sb.append('.').append(AbstractDataChannelBASt.TWO_DIGIT_FORMAT.format(year % 100));
		if (isQuartal) {
			sb.append(monthOrQuartal);
		} else {
			sb.append(AbstractDataChannelBASt.MONATE[monthOrQuartal]);
			DataChannelBAStV20.LOGGER.finer("FileName: %s", sb);
		}
		return sb.toString();
	}

	final private String calculateFirstRow() {
		String sBundesLandNummer = String.valueOf(_ZaehlStelle.getBundesland().getKennung());
		while (sBundesLandNummer.length() == 1) {
			sBundesLandNummer = '0' + sBundesLandNummer;
		}

		final StringBuffer sb = new StringBuffer("H");
		sb.append(_ZaehlStelle.getLzzNummerAsString()).append(' ');
		sb.append(sBundesLandNummer).append(' ');

		sb.append(_ZaehlStelle.getStrassenKlasse().getAbbreviation()).append(' ');

		final Strasse strasse = _ZaehlStelle.getStrasse();
		try {
			sb.append(strasse.getNummer()).append(strasse.getZusatz());
		} catch (final Exception ex1) {
			ex1.printStackTrace();
		}
		while (sb.length() < 21) {
			sb.append(' ');
		}

		String sLzzName = _ZaehlStelle.getLzzName();
		if (sLzzName.length() > 25) {
			sLzzName = sLzzName.substring(0, 24);
		}
		sb.append(_ZaehlStelle.getLzzName());
		while (sb.length() < 46) {
			sb.append(' ');
		}
		sb.append("V2.0;");
		return sb.toString();
	}

	final private String calculateSecondRow() {
		final AuswerteQuerschnitt inAuswertequerschnitt = _ZaehlStelle.getInAuswertequerschnitt();
		final int fsCountInRichtung = inAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;

		final String sFsCountInRichtung = String.valueOf(fsCountInRichtung);
		String inFernziel = _ZaehlStelle.getInFernziel();
		final String sInHimmelsrichtung = inAuswertequerschnitt.getHimmelsrichtung();

		final AuswerteQuerschnitt gegenAuswertequerschnitt = _ZaehlStelle.getInAuswertequerschnitt();
		final int fsCountGegenRichtung = gegenAuswertequerschnitt.getMessQuerschnitt().getFahrstreifen().length;
		String gegenFernziel = _ZaehlStelle.getGegenFernziel();
		final String sGegenHimmelsrichtung = gegenAuswertequerschnitt.getHimmelsrichtung();

		final StringBuffer sb = new StringBuffer("R");
		if (fsCountInRichtung < 10) {
			sb.append('0');
		}
		sb.append(String.valueOf(fsCountInRichtung)).append(' ');
		if (fsCountGegenRichtung < 10) {
			sb.append('0');
		}
		sb.append(String.valueOf(fsCountGegenRichtung)).append(' ');

		if (inFernziel.length() > 20) {
			inFernziel = inFernziel.substring(0, 19);
		}
		sb.append(inFernziel);
		while (sb.length() < 27) {
			sb.append(' ');
		}
		sb.append(sInHimmelsrichtung).append(' ');

		if (gegenFernziel.length() > 20) {
			gegenFernziel = gegenFernziel.substring(0, 19);
		}
		sb.append(gegenFernziel);
		while (sb.length() < 49) {
			sb.append(' ');
		}
		sb.append(sGegenHimmelsrichtung).append(';');
		return sb.toString();
	}

	@Override
	protected void writeHeader(final PrintWriter writer) {
		writer.println(calculateFirstRow());
		writer.println(calculateSecondRow());
		writer.println(_ThirdLine);
	}

	private final static String format(final String s) {
		String retValue = s;
		switch (s.length()) {
		case 2:
			retValue = "    " + s;
			break;
		case 3:
			retValue = "   " + s;
			break;
		case 4:
			retValue = "  " + s;
			break;
		case 5:
			retValue = " " + s;
			break;
		}
		return retValue;
	}

	private interface ValueExtractor {
		String extractData(Dataset[] rsArray);
	}

	/**
	 * extrahiert die Werte aus den ResultData und baut einen String mit den
	 * Werten f&uuml;r die Klassifizierungsgruppe 8plus1.
	 */
	private class ValueExtractor8plus1 implements ValueExtractor {

		@Override
		public String extractData(final Dataset[] rsArray) {

			final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];
			if (rsArray != null) {
				for (int i = 0; i < _CountOfFS; i++) {
					if ((rsArray[i] != null) && (rsArray[i].getData() != null)) {
						atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
					}
				}
			}
			// TLS: KFZ Krad Pkw LfW PkwA Bus Lkw LkwA SattelKfz nkKfz
			// BASt: KFZ SV Mot Pkw LfW PmA Bus LoA LmA Sat Son
			final StringBuffer sb = new StringBuffer();
			// zuerst die Gruppen KFZ, SV für alle Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if (atgLZArray[i] == null) {
					sb.append("    0a    0a");
				} else {
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKfz().getValueAsBastString()));
					final String s = atgLZArray[i]
							.getSVasBastString(AtgVerkehrsdatenLangZeitIntervall.KLASSIFIZIERUNG_8PLUS1);
					sb.append(DataChannelBAStV20.format(s));
				}
			}
			// die anderen Werte Mot, Pkw, LfW, PmA Bus, LoA, LmA, Sat, Son für
			// alle Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if (atgLZArray[i] == null) {
					sb.append("    0a    0a    0a    0a    0a    0a    0a    0a    0a");
				} else {
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKrad().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqPkw().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqLfw().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqPkwA().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqBus().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqLkw().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqLkwA().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqSattelKfz().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKfzNk().getValueAsBastString()));
				}
			}
			return sb.toString();
		}
	}

	/**
	 * extrahiert die Werte aus den ResultData und baut einen String mit den
	 * Werten f&uuml;r die Klassifizierungsgruppe 5plus1.
	 */
	private class ValueExtractor5plus1 implements ValueExtractor {

		@Override
		public String extractData(final Dataset[] rsArray) {
			final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];
			for (int i = 0; i < _CountOfFS; i++) {
				if ((rsArray[i] != null) && (rsArray[i].getData() != null)) {
					atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
				}
			}
			// TLS: KFZ PkwG PkwA Bus Lkw LkwK nkKfz
			// BASt: KFZ SV PLZ PmA Bus LoA Lzg Son
			final StringBuffer sb = new StringBuffer();
			// zuerst die Gruppen KFZ, SV für alle Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if (atgLZArray[i] == null) {
					sb.append("    0a    0a");
				} else {
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKfz().getValueAsBastString()));
					final String s = atgLZArray[i]
							.getSVasBastString(AtgVerkehrsdatenLangZeitIntervall.KLASSIFIZIERUNG_5PLUS1);
					sb.append(DataChannelBAStV20.format(s));
				}
			}

			// die anderen Werte PLZ, PmA, Bus, LoA, Lzg, Son für alle
			// Fahrstreifen
			for (int i = 0; i < _CountOfFS; i++) {
				if (atgLZArray[i] == null) {
					sb.append("    0a    0a    0a    0a    0a    0a");
				} else {
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqPkwG().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqPkwA().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqLkw().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqLkwK().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqBus().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKfzNk().getValueAsBastString()));
				}
			}
			return sb.toString();
		}
	}

	/**
	 * erzeugt einen String mit den Werten f&uuml;r die Klassifizierungsgruppe
	 * PkwAehnliche und LkwAehnliche (2 Gruppe).
	 */
	private class ValueExtractor2 implements ValueExtractor {

		@Override
		public String extractData(final Dataset[] rsArray) {

			final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];
			for (int i = 0; i < _CountOfFS; i++) {
				if ((rsArray[i] != null) && (rsArray[i].getData() != null)) {
					atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
				}
			}
			// TLS: KFZ LkwÄ
			// BASt: KFZ Lkw
			final StringBuffer sb = new StringBuffer();

			for (int i = 0; i < _CountOfFS; i++) {
				if (atgLZArray[i] == null) {
					sb.append("    0a    0a");
				} else {
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKfz().getValueAsBastString()));
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqLkwAE().getValueAsBastString()));
				}
			}
			return sb.toString();
		}
	}

	/**
	 * erzeugt einen String mit den Werten f&uuml;r die Klassifizierungsgruppe
	 * Kfz (1 Gruppe).
	 */
	private class ValueExtractor1 implements ValueExtractor {

		@Override
		public String extractData(final Dataset[] rsArray) {

			final AtgVerkehrsdatenLangZeitIntervall[] atgLZArray = new AtgVerkehrsdatenLangZeitIntervall[_CountOfFS];
			for (int i = 0; i < _CountOfFS; i++) {
				if ((rsArray[i] != null) && (rsArray[i].getData() != null)) {
					atgLZArray[i] = new AtgVerkehrsdatenLangZeitIntervall(rsArray[i].getData());
				}
			}
			// TLS: KFZ
			// BASt: KFZ
			final StringBuffer sb = new StringBuffer();

			for (int i = 0; i < _CountOfFS; i++) {
				if (atgLZArray[i] == null) {
					sb.append("    0a");
				} else {
					sb.append(DataChannelBAStV20.format(atgLZArray[i].getqKfz().getValueAsBastString()));
				}
			}
			return sb.toString();
		}
	}

}
