/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output;

import java.util.Calendar;

import de.bsvrz.dua.bastband.bastband.dvs.funclib.objectwrapper.LangZeitZaehlStelle;

/**
 * History:
 *
 * $Log: DataChannelBASt_MV.java,v $
 * Revision 1.5  2007/07/26 12:49:55  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.4  2007/07/26 08:01:42  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.3  2007/07/25 13:02:06  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2007/07/04 11:48:42  Trier
 * Neuordnung
 *
 * Revision 1.1.1.1  2007/06/20 09:56:27  Trier
 * dvs alles
 *
 * Revision 1.3  2005/02/23 09:15:21  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.2  2005/02/18 10:09:51  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/31 15:16:10  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1  2005/01/21 14:40:07  Trier
 * <Kein Kommentar eingegeben>
 *
 */

/**
 * <p>
 * Channel zur Ausgabe von Daten in das BASt-Bestandsbandformat in der von
 * Sachsen-Anhalt verwendeten Version.
 * </p>
 *
 * <p>
 * Es wird die Attributegrupe atg.verkehrsDatenLangZeitIntervall mit dem Aspekt
 * asp.messWertErsetzung ausgegeben.
 * </p>
 * <p>
 * Die Unterstützung f&uuml;r die durch die BASt definierten Pr&uuml;fzeichen
 * ist der folgenen Tabelle zu entnehmen.
 * </p>
 * <table border=1 align=center cellpadding=2 cellspacing=0 summary="Werte">
 * <tr>
 * <th align=left>Pr&uuml;fkennzeichen</th> <th align=left>Bemerkung</th> </tr>
 * <tr> <td>&nbsp;</td> <td>korrekte, regelmäßige Werte</td> </tr> <tr>
 * <td>'</td> <td>alle anderen Werte</td> </tr> </table>
 *
 * <p> Bei der Umstellung MESZ zur MEZ wird der 2. Datensatz für 02 Uhr
 * ausgegeben. </p>
 *
 * <p> siehe auch:
 * {@link de.bsvrz.dua.bastband.bastband.dvs.kex.datenaustausch.output.AbstractDataChannelBASt
 * AbstractDataChannelBASt} </p>
 *
 */
public class DataChannelBASt_MV extends DataChannelBAStV1_X {

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 */
	public DataChannelBASt_MV() {
		super();
		setCountOfGeschwKlassen(10);
	}

	/**
	 * erzeugt einen DataChannel, der mit einer reset-Methode und eventuell
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss zur Ausgabe der angegebenen SatzArt.
	 */
	public DataChannelBASt_MV(final int satzArt) {
		super(satzArt);
		setCountOfGeschwKlassen(10);
	}

	/**
	 * erzeugt einen DataChannel, der vor dem Aufruf von
	 * {@link #processRecord(de.bsvrz.dav.daf.main.Dataset[], Calendar)} mit
	 * {@link DataChannelBAStV1_X#setZaehlstelle(LangZeitZaehlStelle)}
	 * initialisiert werden muss.
	 *
	 * <p>
	 * Ist die Datei vorhanden wird diese &uuml;berschrieben.
	 * </p>
	 *
	 * @param fileName
	 *            Dateiname mit vollst&auml;ndigem Pfad
	 */
	public DataChannelBASt_MV(final String fileName) throws ChannelException {
		super(fileName);
		setCountOfGeschwKlassen(10);
	}

	/**
	 * erzeugt den f&uuml;r eine Z&auml;hlstelle konstanten Zeilenanfang. Der
	 * Anfang jeder Zeile besteht aus TK-BlattNr, Z&auml;hlstellenNr.,
	 * L&auml;nderkennung.
	 */
	@Override
	protected String calculateLineHeader() {
		final StringBuffer lineHeader = new StringBuffer(8);
		lineHeader.append(_ZaehlStelle.getTK25AsString());
		lineHeader.append(_ZaehlStelle.getBastNummerAsString());
		return lineHeader.toString();
	}

}
