/**
 * Copyright (c) 2004 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.funclib.logger;

/**
 * History:
 *
 * $Log: LogFormatter.java,v $
 * Revision 1.2  2007/08/21 09:06:05  Trier
 * <Kein Kommentar eingegeben>
 *
 * Revision 1.1.1.1  2007/06/14 08:37:53  Trier
 * Allgemeine Funktionsbibliothek
 *
 *
 */
import java.io.PrintWriter;
import java.io.StringWriter;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.logging.Formatter;
import java.util.logging.LogRecord;

/**
 * <p>
 * Klasse zum Formatieren von LogRecord.
 * </p>
 *
 * @author Ralf Trier
 * @version 1.0
 */
public class LogFormatter extends Formatter {

	static final String CRLF = System.getProperty("line.separator");
	static final SimpleDateFormat DATEFORMAT = new SimpleDateFormat(
			"yyyy-MM-dd HH:mm:ss,SSS");

	@Override
	public String format(final LogRecord record) {
		final Date date = new Date(record.getMillis());
		final StringBuilder sb = new StringBuilder();
		sb.append(LogFormatter.DATEFORMAT.format(date));
		sb.append(" Thread=").append(record.getThreadID()).append(' ');
		sb.append(record.getLevel()).append(' ');
		sb.append(record.getLoggerName()).append(" - ");
		sb.append(formatMessage(record));
		if (record.getThrown() != null) {
			try {
				final StringWriter sw = new StringWriter();
				final PrintWriter pw = new PrintWriter(sw);
				record.getThrown().printStackTrace(pw);
				pw.close();
				sb.append(LogFormatter.CRLF);
				sb.append("      ").append(sw.toString());
			} catch (final Exception ex) {
			}
		} else {
			sb.append(LogFormatter.CRLF);
		}
		return sb.toString();
	}

}
