/**
 * Copyright (c) 2007 Logos GmbH Rostock (www.logos-ib.de).
 * All rights reserved.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL  LOGOS GMBH  OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package de.bsvrz.dua.bastband.bastband.funclib.logger;

import java.io.OutputStream;
import java.util.Formatter;
import java.util.logging.LogManager;
import java.util.logging.Logger;

/**
 * History:
 *
 * $Log: StandardLogger.java,v $
 * Revision 1.2  2007/08/06 09:50:19  Trier
 * Änderung Log-System
 *
 * Revision 1.1.1.1  2007/06/14 08:37:53  Trier
 * Allgemeine Funktionsbibliothek
 *
 *
 */

/**
 *
 * <p>
 * Der StandardLogger erweitert den <code>java.util.logging.Logger</code>, um
 * das Interface {@link Logger} zu implementieren.
 * </p>
 *
 * @author Ralf Trier
 */
public class StandardLogger extends java.util.logging.Logger implements
de.bsvrz.dua.bastband.bastband.funclib.logger.Logger {

	private static boolean _NotInitialized = true;

	public static final java.util.logging.Level[] JLEVEL = new java.util.logging.Level[] {
		java.util.logging.Level.FINEST, java.util.logging.Level.INFO,
		java.util.logging.Level.WARNING, java.util.logging.Level.SEVERE,
		java.util.logging.Level.SEVERE };

	StandardLogger(final Class c) {
		super(c.getName(), null);
	}

	StandardLogger(final String s) {
		super(s, null);
	}

	public static StandardLogger getStandardLogger(final Class c) {
		return StandardLogger.getStandardLogger(c.getName());
	}

	public synchronized static StandardLogger getStandardLogger(final String s) {
		final LogManager manager = LogManager.getLogManager();
		StandardLogger result = (StandardLogger) manager.getLogger(s);
		if (result == null) {
			result = new StandardLogger(s);
			manager.addLogger(result);
			result = (StandardLogger) manager.getLogger(s);
		}
		return result;
	}

	@Override
	public void finer(final String format, final Object... args) {
		if (this.isLoggable(java.util.logging.Level.FINER)) {
			try {
				this.log(java.util.logging.Level.FINER, (new Formatter())
						.format(format, args).toString());
			} catch (final Exception e) {
				this.log(java.util.logging.Level.FINER,
						"FormatString nicht in Ordnung: " + format);
			}
		}
	}

	@Override
	public void finer(final Object o) {
		log(java.util.logging.Level.FINER, o);
	}

	@Override
	public void debug(final Object o) {
		log(java.util.logging.Level.FINE, o);
	}

	@Override
	public void info(final Object o) {
		log(java.util.logging.Level.INFO, o);
	}

	@Override
	public void warn(final Object o) {
		log(java.util.logging.Level.WARNING, o);
	}

	@Override
	public void error(final Object o) {
		log(java.util.logging.Level.SEVERE, o);
	}

	@Override
	public void fatal(final Object o) {
		log(java.util.logging.Level.SEVERE, o);
	}

	private void log(final java.util.logging.Level level, final Object o) {
		if (this.isLoggable(level)) {
			if (o == null) {
				this.log(level, "-");
			} else {
				this.log(level, o.toString());
			}
		}
	}

	@Override
	public void finer(final Object o, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.FINER)) {
			String msg;
			if (o == null) {
				msg = "-";
			} else {
				msg = o.toString();
			}
			this.log(java.util.logging.Level.FINER, msg, e);
		}
	}

	@Override
	public void debug(final String format, final Object... args) {
		if (this.isLoggable(java.util.logging.Level.FINE)) {
			try {
				this.log(java.util.logging.Level.FINE, (new Formatter())
						.format(format, args).toString());
			} catch (final Exception e) {
				this.log(java.util.logging.Level.FINE,
						"FormatString nicht in Ordnung: " + format);
			}
		}
	}

	@Override
	public void debug(final Object o, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.FINE)) {
			String msg;
			if (o == null) {
				msg = "-";
			} else {
				msg = o.toString();
			}
			this.log(java.util.logging.Level.FINE, msg, e);
		}
	}

	@Override
	public void warn(final Object o, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.WARNING)) {
			if (o == null) {
				this.log(java.util.logging.Level.WARNING, "-", e);
			} else {
				this.log(java.util.logging.Level.WARNING, o.toString(), e);
			}
		}
	}

	@Override
	public void warn(final String format, final Object... args) {
		if (this.isLoggable(java.util.logging.Level.WARNING)) {
			try {
				this.log(java.util.logging.Level.WARNING, (new Formatter())
						.format(format, args).toString());
			} catch (final Exception e) {
				this.log(java.util.logging.Level.WARNING,
						"FormatString nicht in Ordnung: " + format);
			}
		}
	}

	public void info(final Object o, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.INFO)) {
			if (o == null) {
				this.log(java.util.logging.Level.INFO, "-", e);
			} else {
				this.log(java.util.logging.Level.INFO, o.toString(), e);
			}
		}
	}

	@Override
	public void info(final String s, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.INFO)) {
			this.log(java.util.logging.Level.INFO, s, e);
		}
	}

	@Override
	public void info(final String format, final Object... args) {
		if (this.isLoggable(java.util.logging.Level.INFO)) {
			try {
				this.log(java.util.logging.Level.INFO, (new Formatter())
						.format(format, args).toString());
			} catch (final Exception e) {
				this.log(java.util.logging.Level.INFO,
						"FormatString nicht in Ordnung: " + format);
			}
		}
	}

	@Override
	public void error(final String cause, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.WARNING)) {
			this.log(java.util.logging.Level.WARNING, cause, e);
		}
	}

	@Override
	public void error(final String format, final Object... args) {
		if (this.isLoggable(java.util.logging.Level.WARNING)) {
			try {
				this.log(java.util.logging.Level.WARNING, (new Formatter())
						.format(format, args).toString());
			} catch (final Exception e) {
				this.log(java.util.logging.Level.WARNING,
						"FormatString nicht in Ordnung: " + format);
			}
		}
	}

	@Override
	public void fatal(final String cause, final Exception e) {
		if (this.isLoggable(java.util.logging.Level.SEVERE)) {
			this.log(java.util.logging.Level.SEVERE, cause, e);
		}
	}

	@Override
	public void fatal(final String format, final Object... args) {
		if (this.isLoggable(java.util.logging.Level.SEVERE)) {
			try {
				this.log(java.util.logging.Level.SEVERE, (new Formatter())
						.format(format, args).toString());
			} catch (final Exception e) {
				this.log(java.util.logging.Level.SEVERE,
						"FormatString nicht in Ordnung: " + format);
			}
		}
	}

	@Override
	public void setLevel(
			final de.bsvrz.dua.bastband.bastband.funclib.logger.Logger.Level level) {
		this.setLevel(StandardLogger.JLEVEL[level.ordinal()]);
	}

	@Override
	public void setOutput(final OutputStream out) {
		throw new IllegalArgumentException("Methode ist nicht implementiert");
	}

}
